/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LANE_H
#define DPC_IMPL_TYPE_LANE_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_neighborlane.h"
#include "dpc/impl_type_vector_uint32.h"
#include "dpc/impl_type_vector_corridor.h"
#include "dpc/impl_type_vector_samplingpoint.h"
#include "dpc/impl_type_vector_lanecontrol.h"
#include "impl_type_boolean.h"

namespace dpc {
struct Lane {
    ::UInt8 start_id{ 0U };
    ::UInt8 end_id{ 0U };
    ::Float start_range{ 0.0F };
    ::Float end_range{ 0.0F };
    ::dpc::vector_NeighborLane left_neighbor_lane;
    ::dpc::vector_NeighborLane right_neighbor_lane;
    ::dpc::vector_uint32 left_lane_marker_id_list;
    ::dpc::vector_uint32 right_lane_marker_id_list;
    ::dpc::vector_Corridor corridor_list;
    ::dpc::vector_Corridor back_corridor_list;
    ::dpc::vector_SamplingPoint sampling_point_list;
    ::UInt8 lane_type{ 0U };
    ::dpc::vector_LaneControl max_speed_limit;
    ::Float lane_width{ 0.0F };
    ::Boolean is_start_out_of_range{ false };
    ::Boolean is_end_out_of_range{ false };
    ::UInt8 lane_start_split_merge_type{ 0U };
    ::UInt8 lane_end_split_merge_type{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(start_id);
        fun(end_id);
        fun(start_range);
        fun(end_range);
        fun(left_neighbor_lane);
        fun(right_neighbor_lane);
        fun(left_lane_marker_id_list);
        fun(right_lane_marker_id_list);
        fun(corridor_list);
        fun(back_corridor_list);
        fun(sampling_point_list);
        fun(lane_type);
        fun(max_speed_limit);
        fun(lane_width);
        fun(is_start_out_of_range);
        fun(is_end_out_of_range);
        fun(lane_start_split_merge_type);
        fun(lane_end_split_merge_type);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(start_id);
        fun(end_id);
        fun(start_range);
        fun(end_range);
        fun(left_neighbor_lane);
        fun(right_neighbor_lane);
        fun(left_lane_marker_id_list);
        fun(right_lane_marker_id_list);
        fun(corridor_list);
        fun(back_corridor_list);
        fun(sampling_point_list);
        fun(lane_type);
        fun(max_speed_limit);
        fun(lane_width);
        fun(is_start_out_of_range);
        fun(is_end_out_of_range);
        fun(lane_start_split_merge_type);
        fun(lane_end_split_merge_type);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("start_id", start_id);
        fun("end_id", end_id);
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("left_neighbor_lane", left_neighbor_lane);
        fun("right_neighbor_lane", right_neighbor_lane);
        fun("left_lane_marker_id_list", left_lane_marker_id_list);
        fun("right_lane_marker_id_list", right_lane_marker_id_list);
        fun("corridor_list", corridor_list);
        fun("back_corridor_list", back_corridor_list);
        fun("sampling_point_list", sampling_point_list);
        fun("lane_type", lane_type);
        fun("max_speed_limit", max_speed_limit);
        fun("lane_width", lane_width);
        fun("is_start_out_of_range", is_start_out_of_range);
        fun("is_end_out_of_range", is_end_out_of_range);
        fun("lane_start_split_merge_type", lane_start_split_merge_type);
        fun("lane_end_split_merge_type", lane_end_split_merge_type);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("start_id", start_id);
        fun("end_id", end_id);
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("left_neighbor_lane", left_neighbor_lane);
        fun("right_neighbor_lane", right_neighbor_lane);
        fun("left_lane_marker_id_list", left_lane_marker_id_list);
        fun("right_lane_marker_id_list", right_lane_marker_id_list);
        fun("corridor_list", corridor_list);
        fun("back_corridor_list", back_corridor_list);
        fun("sampling_point_list", sampling_point_list);
        fun("lane_type", lane_type);
        fun("max_speed_limit", max_speed_limit);
        fun("lane_width", lane_width);
        fun("is_start_out_of_range", is_start_out_of_range);
        fun("is_end_out_of_range", is_end_out_of_range);
        fun("lane_start_split_merge_type", lane_start_split_merge_type);
        fun("lane_end_split_merge_type", lane_end_split_merge_type);
    }

    friend bool operator==(const ::dpc::Lane& lhs, const ::dpc::Lane& rhs) noexcept
    {
        return (lhs.start_id == rhs.start_id) && (lhs.end_id == rhs.end_id) && (fabs(static_cast<double>(lhs.start_range - rhs.start_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_range - rhs.end_range)) < DBL_EPSILON) && (lhs.left_neighbor_lane == rhs.left_neighbor_lane) && (lhs.right_neighbor_lane == rhs.right_neighbor_lane) && (lhs.left_lane_marker_id_list == rhs.left_lane_marker_id_list) && (lhs.right_lane_marker_id_list == rhs.right_lane_marker_id_list) && (lhs.corridor_list == rhs.corridor_list) && (lhs.back_corridor_list == rhs.back_corridor_list) && (lhs.sampling_point_list == rhs.sampling_point_list) && (lhs.lane_type == rhs.lane_type) && (lhs.max_speed_limit == rhs.max_speed_limit) && (fabs(static_cast<double>(lhs.lane_width - rhs.lane_width)) < DBL_EPSILON) && (lhs.is_start_out_of_range == rhs.is_start_out_of_range) && (lhs.is_end_out_of_range == rhs.is_end_out_of_range) && (lhs.lane_start_split_merge_type == rhs.lane_start_split_merge_type) && (lhs.lane_end_split_merge_type == rhs.lane_end_split_merge_type);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LANE_H
